@@ -1,8 +1,8 @@
-from dagster.core.utils import check_dagster_package_version
+from dagster._core.utils import check_dagster_package_version
 
 from .context import DagstermillExecutionContext
 from .errors import DagstermillError
-from .factory import define_dagstermill_op, define_dagstermill_solid
+from .factory import define_dagstermill_op
 from .io_managers import local_output_notebook_io_manager
 from .manager import MANAGER_FOR_NOTEBOOK_INSTANCE as _MANAGER_FOR_NOTEBOOK_INSTANCE
 from .version import __version__
